/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org view-source front-end.
 *
 * The Initial Developer of the Original Code is mozilla.org.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Jeroen Groenenboom <jagrboom@zonnet.nl> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function Doc() {
    this.string = '<html>\n<head>\n<title>BookmarksHome</title>' +
	'\n<script  type="text/javascript" src="chrome://bookmarkshome/content/bookmarksHomePrefs.js"></script>' +
	'\n<style>\n' + bMHPrefs.getPref("stylesheet") + '\n</style>\n</head>\n<body>\n<div align="center">';
}

Doc.prototype = {

add: function( html ) {
	 this.string += html;
     },

close: function() {
	   this.string += ( '<a href=\'javascript:void(w=window.open("bookmarksHomePrefsWindow.html",' +
			   '"prefs","height=550,width=475,resizable=yes,scrollbars=yes"));w.focus();\'>options</a>' + 
		   '\n</div>\n</body>\n</html>' );
       },
}

function SearchTable() {
    var engineIndex = bMHPrefs.getPref( "engine" );
    if( bMHPrefs.engines[engineIndex].value == "none" ) {
	this.string = "";
    }
    else {
	this.string = '\n<form id="searchForm" onsubmit="javascript:return bMHPrefs.internetSearch();">' +
	    '\n<table>\n<tr>\n<td><select id="engineSelect">' + bMHPrefs.engineOptions( "main" ) + 
	    '\n</select><td>\n<td><input type="text" tabindex="0" id="queryInput" size="31"></td>' +
	    '\n<td><input type="submit" value="Search"></td>\n</tr>\n</table>\n</form>' +
	    '\n<script type="text/javascript">\nsetTimeout(\'document.getElementById("queryInput").focus()\',0);' +
	    '\ndocument.getElementById("engineSelect").selectedIndex=bMHPrefs.getPref("engine");\n</script>';
    }
}


function MainTable( columns ) {
    this.string = '\n<table class="table-main" cellpadding="' + 
	bMHPrefs.getPref( "table-main-cellpadding" ) + '">\n<tr>';
}

MainTable.prototype = {

add: function( element ) {
	 this.string += '\n<td valign="top">' + element.string + '</td>';
     },

close: function() {
	   this.string += '\n</tr>\n</table>';
       },
}

function ColumnTable() {
    this.string = '\n<table class="table-column" cellspacing="' +
	bMHPrefs.getPref( "table-column-cellspacing" ) + '">';
    this.depth = 0;
}

ColumnTable.prototype = {

add: function( folder ) {
	 this.string += '\n<tr><td>' + folder.string + '</td></tr>';
	 this.depth += ( folder.depth + 1 );
     },

close: function() {
	   this.string += '\n</table>';
       },
}

function FolderTable( ancestors ) {
    this.string = '\n<table class="table-folder" width="100%">';
    this.depth = 0;
    var name, title;
    for( var i = 1; i < ancestors.length; i++ ) {
	title = this.getTitle( ancestors[i] );
	name = this.processName( ancestors[i] );	
	this.string += '\n<tr><th align="left" ' + title +
	    '>' + name + '</tr></th>';
	this.depth++;
    }
    this.startDepth = this.depth;
}


FolderTable.prototype = {

add: function( name, url ) {
	 var title = this.getTitle( name );
	 name = this.processName( name );
	 this.string += '\n<tr><td><a href="' + url +
	     '" ' + title + '>' + name + '</a></td></tr>';
	 this.depth++;
     },

getTitle: function( name ) {
	      return name.length > bMHPrefs.getPref( "maxNameLength" )
		  ? 'title="' + name + '"'
		  : '';
	  },

processName: function( name ) {
		 var reg1 = /^(?:\w+:[\/\\]{1,3}(?:www\.)?)/; 
		 var reg2 = /.{2}$/;	
		 max = bMHPrefs.getPref( "maxNameLength" );
		 if( name.length > max ) {
		     // remove protocol of url
		     name = name.replace( reg1, "" );
		     if( name.length > max ) {
			 name = name.substring( 0, max );
			 // show that name is longer than portrayed
			 name = name.replace( reg2, ".." );
		     }
		 }
		 return name;
	     },

close: function() {
	   this.string += "\n</table>";
       },
}

function Folder( node, ancestors ) {
    this.node = node;
    // ancestors include name of folder
    this.ancestors = ancestors;
}

function FolderTree() {
    this.string = '\n<form id="hiddenFolderForm">';
}

FolderTree.prototype = {

add: function( ancestorsString ) {
	 this.string += '\n<input type="hidden" name="folderInput" value="' + ancestorsString + '"</input>'
     },

close: function() {
	   this.string += '\n</form>'
       },
}

var bookmarksHome = {

main: function() {
	  // RDF variables
	  var NC_NS = "http://home.netscape.com/NC-rdf#";

	  var kRDFContractID = "@mozilla.org/rdf/rdf-service;1";
	  var kRDFSVCIID = Components.interfaces.nsIRDFService;
	  var kRDFRSCIID = Components.interfaces.nsIRDFResource;
	  var kRDFLITIID = Components.interfaces.nsIRDFLiteral;
	  var RDF = Components.classes[kRDFContractID].getService( kRDFSVCIID );

	  var kRDFCContractID = "@mozilla.org/rdf/container;1";
	  var kRDFCIID = Components.interfaces.nsIRDFContainer;
	  var RDFC = Components.classes[kRDFCContractID].createInstance( kRDFCIID );
	  var kRDFCUContractID = "@mozilla.org/rdf/container-utils;1";
	  var kRDFCUIID = Components.interfaces.nsIRDFContainerUtils;
	  var RDFCU = Components.classes[kRDFCUContractID].getService(kRDFCUIID);

	  var kBMSVCIID = Components.interfaces.nsIBookmarksService;
	  var BMDS  = RDF.GetDataSource("rdf:bookmarks");
	  var root = RDF.GetResource( "NC:BookmarksRoot" );
	  var BMSVC = BMDS.QueryInterface( kBMSVCIID );
	  var NameArc = RDF.GetResource( NC_NS+"Name" );
	  var URLArc =  RDF.GetResource( NC_NS+"URL" );

	  // initialize preferences
	  bMHPrefs.init();

	  // bookmarks collection variables
	  var curNameNode = BMDS.GetTarget( root, NameArc, true );
	  // necessary typecast
	  if( curNameNode instanceof kRDFLITIID );
	  var curFolder = new Folder( root, [curNameNode.Value] );
	  var folderStack = new Array( curFolder );
	  var inOrExcludedFolders = bMHPrefs.getPref( "inOrExcludedFolders" ).split( "|" );
	  var folderTree = new FolderTree();
	  folderTree.add( curFolder.ancestors ); 
	  var enumerator, tempFolders, curNode, curUrlNode, curNodeAncestors;

	  // html variables
	  var doc = new Doc();
	  var searchTable = new SearchTable();
	  var columns = bMHPrefs.getPref( "nrOfColumns" ); 
	  var mainTable = new MainTable( columns ); 
	  var columnTables = new Array( columns );
	  for( i = 0; i < columnTables.length; i++ ) {
	      columnTables[i] = new ColumnTable();
	  }
	  var folderTable;

	  var isExcluded, i;
	  // make page
	  while( folderStack.length > 0 ) {
	      curFolder = folderStack.pop();
	      folderTable = new FolderTable( curFolder.ancestors );
	      RDFC.Init( BMDS, curFolder.node );
	      enumerator = RDFC.GetElements();
	      tempFolders = new Array();
	      while( enumerator.hasMoreElements() ) {
		  curNode = enumerator.getNext();
		  curNameNode = BMDS.GetTarget( curNode, NameArc, true );
		  curUrlNode = BMDS.GetTarget( curNode, URLArc, true );
		  if( curNameNode instanceof kRDFLITIID ) { 
		      if( curUrlNode instanceof kRDFLITIID ) { 
			  // curNameNode is bookmark
			  folderTable.add( curNameNode.Value, curUrlNode.Value );
		      }
		      else if( RDFCU.IsSeq( BMDS, curNode )) {
			  // curNameNode is folder
			  curNodeAncestors = curFolder.ancestors.concat( curNameNode.Value );
			  folderTree.add( curNodeAncestors.toString() );
			  tempFolders.push( new Folder( curNode, curNodeAncestors ));
		      }
		  }
	      }
	      folderTable.close();
	      isExcluded = bMHPrefs.isExcludedFolder( curFolder.ancestors.toString(),
		      bMHPrefs.getPref( "exclude" ), inOrExcludedFolders );
	      // check if folder contains bookmarks and is not excluded
	      if( folderTable.depth > folderTable.startDepth && ! isExcluded ) {
		  i = bookmarksHome.smallestColumn( columnTables );
		  columnTables[i].add( folderTable );
	      }
	      // reverse to match order in bookmarks tree
	      folderStack = folderStack.concat( tempFolders.reverse() );
	  }
	  folderTree.close();
	  for( i = 0; i < columnTables.length; i++ ) {
	      columnTables[i].close();
	      mainTable.add( columnTables[i] );
	  }
	  mainTable.close();
	  doc.add( searchTable.string );
	  doc.add( mainTable.string );
	  doc.add( folderTree.string );
	  doc.close();
	  document.write( doc.string );
	  document.close();
      },

smallestColumn: function( columns ) {
		    var index = 0;
		    for(var i = 1; i < columns.length; i++ ) {
			if( columns[i].depth < columns[index].depth ) {
			    index = i;
			}
		    }
		    return index;
		},
}



